import importlib.resources as pkg_resources
import os

import mne

import os
print(os.getcwd())
from alice_ml import data
from alice_ml.features import get_features_from_mne


def test_get_features_from_mne():
    
    raw = mne.io.read_raw_fif(pkg_resources.open_binary(data, 'test_raw.fif'), preload=True, verbose=False)
    
    # TODO: stick to alice_ml path
    path_to_ica = os.path.join(os.getcwd(), 'alice_ml', 'data', 'test_ica.fif')
    ica = mne.preprocessing.read_ica(path_to_ica)

    feature_df = get_features_from_mne(raw, ica)

    # 15 ics in the test sample
    assert feature_df.shape[0] == 15
    # 11 main features
    assert feature_df.shape[1] == 11


import numpy as np
import mne
import pandas as pd


def calc_MI(X,Y,bins=100):
    c_XY = np.histogram2d(X,Y,bins)[0]
    c_X = np.histogram(X,bins)[0]
    c_Y = np.histogram(Y,bins)[0]

    H_X = shan_entropy(c_X)
    H_Y = shan_entropy(c_Y)
    H_XY = shan_entropy(c_XY)

    MI = H_X + H_Y - H_XY
    return MI


def shan_entropy(c):
    c_normalized = c / float(np.sum(c))
    c_normalized = c_normalized[np.nonzero(c_normalized)]
    H = -sum(c_normalized* np.log2(c_normalized))  
    return H


def mean_pairwise_MI(ica,data,bins=100):
    ica_df = ica.get_sources(data).to_data_frame()
    ic_names = [col for col in ica_df.columns if col not in ('epoch', 'time', 'condition')]
    
    signal = ica_df[ic_names].values.T
    M = np.zeros((signal.shape[0],signal.shape[0]))
    for i in range(signal.shape[0]):
        for j in range(signal.shape[0]):
            M[i,j] = calc_MI(signal[i],signal[j],bins)
    return M.mean()


import numpy as np
import pandas as pd
import mne


# TODO Import from existing project's vis.py
def get_epochs_from_df(ica_data, sfreq):
    epochs = ica_data.index
    n_epochs = len(epochs)
    counts = ica_data.apply(len)
    n_samples = counts[epochs[0]]
    assert np.all(counts == n_samples)

    np_data = np.empty((n_epochs, 1, n_samples))

    for idx, epoch in enumerate(epochs):
        np_data[idx, 0, :] = ica_data[epoch]

    info = mne.io.meas_info.create_info(['ICA000'], sfreq=sfreq, ch_types="misc", verbose=False)

    epochs_from_df = mne.EpochsArray(np_data, info, verbose=False)

    return epochs_from_df


def trim(series, q):
    return series[series < series.quantile(q)]